@@ -10,11 +10,12 @@ import logging
 import os.path
 import sys
 import threading
+import warnings
 
 from django.conf import settings
 from django.contrib.auth.models import User
 from django.core import mail
-from django.core.handlers.wsgi import WSGIRequest, WSGIHandler
+from django.core.handlers.wsgi import WSGIHandler
 from django.core.management import call_command
 from django.core.urlresolvers import reverse
 from django.core.signals import got_request_exception
@@ -83,7 +84,8 @@ def conditional_on_module(module):
             try:
                 __import__(module)
             except ImportError:
-                print "Skipping test: %s.%s" % (self.__class__.__name__, func.__name__)
+                warnings.warn("Skipping test: %s.%s" % (self.__class__.__name__, func.__name__), ImportWarning)
+                return lambda x, *a, **kw: None
             else:
                 return func(self, *args, **kwargs)
         return inner
@@ -705,6 +707,48 @@ class SentryTestCase(TestCase):
         self.assertTrue('baz' in last.data)
         self.assertEquals(last.data['baz'], 'bar')
 
+    def testRawPostData(self):
+        from sentry.reporter import FakeRequest
+        
+        request = FakeRequest()
+        request.raw_post_data = '{"json": "string"}'
+        
+        logger = logging.getLogger()
+
+        self.setUpHandler()
+
+        logger.error('This is a test %s', 'error', extra={
+            'request': request,
+            'data': {
+                'baz': 'bar',
+            }
+        })
+        self.assertEquals(Message.objects.count(), 1)
+        self.assertEquals(GroupedMessage.objects.count(), 1)
+        last = Message.objects.get()
+        self.assertEquals(last.logger, 'root')
+        self.assertEquals(last.level, logging.ERROR)
+        self.assertEquals(last.message, 'This is a test error')
+        self.assertTrue('POST' in last.data)
+        self.assertEquals(request.raw_post_data, last.data['POST'])
+
+    def testScoreUpdate(self):
+        self.assertRaises(Exception, self.client.get, reverse('sentry-raise-exc'))
+        
+        self.assertEquals(GroupedMessage.objects.count(), 1)
+        group = GroupedMessage.objects.get()
+        self.assertTrue(group.score > 0, group.score)
+
+        # drop the score to ensure its getting re-set
+        group.score = 0
+        group.save()
+        
+        self.assertRaises(Exception, self.client.get, reverse('sentry-raise-exc'))
+        self.assertEquals(GroupedMessage.objects.count(), 1)
+
+        group = GroupedMessage.objects.get()
+        self.assertTrue(group.score > 0, group.score)
+
 class SentryViewsTest(TestCase):
     urls = 'sentry.tests.urls'
     fixtures = ['sentry/tests/fixtures/views.json']
@@ -755,13 +799,13 @@ class SentryViewsTest(TestCase):
 
     def testDashboard(self):
         self.client.login(username='admin', password='admin')
-        resp = self.client.get(reverse('sentry'), follow=True)
+        resp = self.client.get(reverse('sentry') + '?sort=freq', follow=True)
         self.assertEquals(resp.status_code, 200)
         self.assertTemplateUsed(resp, 'sentry/index.html')
+        self.assertEquals(len(resp.context['message_list']), 4)
         group = resp.context['message_list'][0]
         self.assertEquals(group.times_seen, 7)
         self.assertEquals(group.class_name, 'AttributeError')
-        self.assertEquals(len(resp.context['message_list']), 4)
 
     def testGroup(self):
         self.client.login(username='admin', password='admin')
@@ -1029,6 +1073,7 @@ class SentryClientTest(TestCase):
     
     def test_get_client(self):
         from sentry.client.log import LoggingSentryClient
+
         self.assertEquals(get_client().__class__, SentryClient)
         self.assertEquals(get_client(), get_client())
     
@@ -1059,8 +1104,10 @@ class SentryClientTest(TestCase):
         self.assertEquals(_foo[''].levelno, client.default_level)
         self.assertEquals(_foo[''].class_name, 'Exception')
 
+    @conditional_on_module('djcelery')
     def test_celery_client(self):
         from sentry.client.celery import CelerySentryClient
+
         self.assertEquals(get_client().__class__, SentryClient)
         self.assertEquals(get_client(), get_client())
 
@@ -1076,6 +1123,26 @@ class SentryClientTest(TestCase):
         self.assertEqual(message.message, 'view exception')
 
         conf.CLIENT = 'sentry.client.base.SentryClient'
+
+    # XXX: need to fix behavior with threads so this test works correctly
+    # def test_async_client(self):
+    #     from sentry.client.async import AsyncSentryClient
+    # 
+    #     self.assertEquals(get_client().__class__, SentryClient)
+    #     self.assertEquals(get_client(), get_client())
+    # 
+    #     conf.CLIENT = 'sentry.client.async.AsyncSentryClient'
+    # 
+    #     self.assertEquals(get_client().__class__, AsyncSentryClient)
+    #     self.assertEquals(get_client(), get_client())
+    # 
+    #     self.assertRaises(Exception, self.client.get, reverse('sentry-raise-exc'))
+    # 
+    #     message = GroupedMessage.objects.get()
+    #     self.assertEqual(message.class_name, 'Exception')
+    #     self.assertEqual(message.message, 'view exception')
+    # 
+    #     conf.CLIENT = 'sentry.client.base.SentryClient'
         
 class SentryManageTest(TestCase):
     fixtures = ['sentry/tests/fixtures/cleanup.json']
@@ -1093,6 +1160,7 @@ class SentryManageTest(TestCase):
 class SentrySearchTest(TestCase):
     urls = 'sentry.tests.urls'
 
+    @conditional_on_module('haystack')
     def test_build_index(self):
         from sentry.views import get_search_query_set
         logger.error('test search error')
